@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2016-2023 CERN
- * Copyright (C) 2016-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
@@ -62,8 +62,7 @@
 
 
 // Reporter is stored by pointer in KIBIS, so keep this here to avoid crashes
-static wxString           s_errors;
-static WX_STRING_REPORTER s_reporter( &s_errors );
+static WX_STRING_REPORTER s_reporter;
 
 
 class SIM_THREAD_REPORTER : public SIMULATOR_REPORTER
@@ -160,7 +159,7 @@ SIMULATOR_FRAME::SIMULATOR_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     m_reporter = new SIM_THREAD_REPORTER( this );
     m_simulator->SetReporter( m_reporter );
 
-    m_circuitModel = std::make_shared<SPICE_CIRCUIT_MODEL>( &m_schematicFrame->Schematic(), this );
+    m_circuitModel = std::make_shared<SPICE_CIRCUIT_MODEL>( &m_schematicFrame->Schematic() );
 
     setupTools();
     setupUIConditions();
@@ -277,6 +276,16 @@ void SIMULATOR_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 }
 
 
+void SIMULATOR_FRAME::CommonSettingsChanged( int aFlags )
+{
+    KIWAY_PLAYER::CommonSettingsChanged( aFlags );
+
+    auto* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( m_toolManager->GetSettings() );
+    wxASSERT( cfg != nullptr );
+    m_ui->ApplyPreferences( cfg->m_Simulator.preferences );
+}
+
+
 WINDOW_SETTINGS* SIMULATOR_FRAME::GetWindowSettings( APP_SETTINGS_BASE* aCfg )
 {
     EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( aCfg );
@@ -334,7 +343,7 @@ void SIMULATOR_FRAME::UpdateTitle()
     if( unsaved )
         title += wxS( " " ) + _( "[Unsaved]" );
 
-    title += wxT( " \u2014 " ) + _( "Spice Simulator" );
+    title += wxT( " \u2014 " ) + _( "SPICE Simulator" );
 
     SetTitle( title );
 }
@@ -343,9 +352,12 @@ void SIMULATOR_FRAME::UpdateTitle()
 // Don't let the dialog grow too tall: you may not be able to get to the OK button
 #define MAX_MESSAGES 20
 
-void SIMULATOR_FRAME::showNetlistErrors( const wxString& aErrors )
+void SIMULATOR_FRAME::showNetlistErrors( const WX_STRING_REPORTER& aReporter )
 {
-    wxArrayString lines = wxSplit( aErrors, '\n' );
+    if( !aReporter.HasMessage() )
+        return;
+
+    wxArrayString lines = wxSplit( aReporter.GetMessages(), '\n' );
 
     if( lines.size() > MAX_MESSAGES )
     {
@@ -353,13 +365,22 @@ void SIMULATOR_FRAME::showNetlistErrors( const wxString& aErrors )
         lines.Add( wxS( "..." ) );
     }
 
-    DisplayErrorMessage( this, _( "Errors during netlist generation." ), wxJoin( lines, '\n' ) );
+    if( aReporter.HasMessageOfSeverity( RPT_SEVERITY_UNDEFINED | RPT_SEVERITY_ERROR ) )
+    {
+        DisplayErrorMessage( this, _( "Errors during netlist generation." ),
+                             wxJoin( lines, '\n' ) );
+    }
+    else if( aReporter.HasMessageOfSeverity( RPT_SEVERITY_WARNING ) )
+    {
+        DisplayInfoMessage( this, _( "Warnings during netlist generation." ),
+                             wxJoin( lines, '\n' ) );
+    }
 }
 
 
 bool SIMULATOR_FRAME::LoadSimulator( const wxString& aSimCommand, unsigned aSimOptions )
 {
-    s_errors.clear();
+    s_reporter.Clear();
 
     if( !m_schematicFrame->ReadyToNetlist( _( "Simulator requires a fully annotated schematic." ) ) )
         return false;
@@ -368,26 +389,23 @@ bool SIMULATOR_FRAME::LoadSimulator( const wxString& aSimCommand, unsigned aSimO
     if( ADVANCED_CFG::GetCfg().m_IncrementalConnectivity )
         m_schematicFrame->RecalculateConnections( nullptr, GLOBAL_CLEANUP );
 
-    if( !m_simulator->Attach( m_circuitModel, aSimCommand, aSimOptions, Prj().GetProjectPath(),
-                              s_reporter ) )
-    {
-        showNetlistErrors( s_errors );
-        return false;
-    }
+    bool success = m_simulator->Attach( m_circuitModel, aSimCommand, aSimOptions,
+                                        Prj().GetProjectPath(), s_reporter );
 
-    return true;
+    showNetlistErrors( s_reporter );
+
+    return success;
 }
 
 
 void SIMULATOR_FRAME::ReloadSimulator( const wxString& aSimCommand, unsigned aSimOptions )
 {
-    s_errors.clear();
+    s_reporter.Clear();
 
-    if( !m_simulator->Attach( m_circuitModel, aSimCommand, aSimOptions, Prj().GetProjectPath(),
-                              s_reporter ) )
-    {
-        showNetlistErrors( s_errors );
-    }
+    m_simulator->Attach( m_circuitModel, aSimCommand, aSimOptions, Prj().GetProjectPath(),
+                         s_reporter );
+
+    showNetlistErrors( s_reporter );
 }
 
 
@@ -535,7 +553,7 @@ bool SIMULATOR_FRAME::LoadWorkbook( const wxString& aPath )
 
         // Successfully loading a workbook does not count as modifying it.  Clear the modified
         // flag after all the EVT_WORKBOOK_MODIFIED events have been processed.
-        CallAfter( [=]()
+        CallAfter( [this]()
                    {
                        m_workbookModified = false;
                    } );
@@ -573,16 +591,15 @@ bool SIMULATOR_FRAME::EditAnalysis()
     SIM_TAB*           simTab = m_ui->GetCurrentSimTab();
     DIALOG_SIM_COMMAND dlg( this, m_circuitModel, m_simulator->Settings() );
 
-    s_errors.clear();
+    s_reporter.Clear();
 
     if( !simTab )
         return false;
 
-    if( !m_circuitModel->ReadSchematicAndLibraries( NETLIST_EXPORTER_SPICE::OPTION_DEFAULT_FLAGS,
-                                                    s_reporter ) )
-    {
-        showNetlistErrors( s_errors );
-    }
+    m_circuitModel->ReadSchematicAndLibraries( NETLIST_EXPORTER_SPICE::OPTION_DEFAULT_FLAGS,
+                                               s_reporter );
+
+    showNetlistErrors( s_reporter );
 
     dlg.SetSimCommand( simTab->GetSimCommand() );
     dlg.SetSimOptions( simTab->GetSimOptions() );
@@ -726,6 +743,8 @@ void SIMULATOR_FRAME::setupUIConditions()
 
     mgr->SetConditions( EE_ACTIONS::exportPlotAsPNG,       ENABLE( havePlot ) );
     mgr->SetConditions( EE_ACTIONS::exportPlotAsCSV,       ENABLE( havePlot ) );
+    mgr->SetConditions( EE_ACTIONS::exportPlotToClipboard, ENABLE( havePlot ) );
+    mgr->SetConditions( EE_ACTIONS::exportPlotToSchematic, ENABLE( havePlot ) );
 
     mgr->SetConditions( ACTIONS::zoomUndo,                 ENABLE( haveZoomUndo ) );
     mgr->SetConditions( ACTIONS::zoomRedo,                 ENABLE( haveZoomRedo ) );
