@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2023 CERN
- * Copyright (C) 2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,9 +26,11 @@
 #include <kiface_base.h>
 #include <kiway.h>
 #include <kiway_express.h>
+#include <board.h>
 #include <wx/button.h>
 #include <wx/checkbox.h>
 #include <kiplatform/ui.h>
+#include <lset.h>
 #include <widgets/panel_footprint_chooser.h>
 #include <settings/settings_manager.h>
 #include <footprint_editor_settings.h>
@@ -37,6 +39,7 @@
 #include <3d_canvas/eda_3d_canvas.h>
 #include <project_pcb.h>
 #include <widgets/bitmap_button.h>
+#include <3d_viewer/eda_3d_viewer_frame.h>
 
 
 static wxArrayString s_FootprintHistoryList;
@@ -69,7 +72,7 @@ END_EVENT_TABLE()
 
 
 #define MODAL_FRAME ( wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN \
-                      | wxWANTS_CHARS | wxFRAME_NO_TASKBAR | wxSTAY_ON_TOP )
+                      | wxWANTS_CHARS | wxFRAME_NO_TASKBAR | wxFRAME_FLOAT_ON_PARENT )
 
 
 FOOTPRINT_CHOOSER_FRAME::FOOTPRINT_CHOOSER_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
@@ -87,23 +90,11 @@ FOOTPRINT_CHOOSER_FRAME::FOOTPRINT_CHOOSER_FRAME( KIWAY* aKiway, wxWindow* aPare
     SetModal( true );
 
     m_showFpMode = true;
+    m_show3DMode = false;
     m_messagePanel->Hide();
 
     wxPanel*    bottomPanel = new wxPanel( this );
     wxBoxSizer* bottomSizer = new wxBoxSizer( wxVERTICAL );
-
-    m_filterByFPFilters = new wxCheckBox( bottomPanel, wxID_ANY, _( "Apply footprint filters" ) );
-    m_filterByPinCount = new wxCheckBox( bottomPanel, wxID_ANY, _( "Filter by pin count" ) );
-
-    m_filterByFPFilters->Show( false );
-    m_filterByPinCount->Show( false );
-
-    if( PCBNEW_SETTINGS* cfg = dynamic_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() ) )
-    {
-        m_filterByFPFilters->SetValue( cfg->m_FootprintChooser.use_fp_filters );
-        m_filterByPinCount->SetValue( cfg->m_FootprintChooser.filter_on_pin_count );
-    }
-
     wxBoxSizer* frameSizer = new wxBoxSizer( wxVERTICAL );
 
     m_chooserPanel = new PANEL_FOOTPRINT_CHOOSER( this, this, s_FootprintHistoryList,
@@ -126,37 +117,41 @@ FOOTPRINT_CHOOSER_FRAME::FOOTPRINT_CHOOSER_FRAME( KIWAY* aKiway, wxWindow* aPare
 
     frameSizer->Add( m_chooserPanel, 1, wxEXPAND );
 
+    SetBoard( new BOARD() );
+
+    // This board will only be used to hold a footprint for viewing
+    GetBoard()->SetBoardUse( BOARD_USE::FPHOLDER );
+
     build3DCanvas();    // must be called after creating m_chooserPanel
     m_preview3DCanvas->Show( !m_showFpMode );
 
-    // The m_filterByFPFilters can have a long string, if the symbol has a FP filter
-    // with many items, so give it its own sizer
-    wxBoxSizer* fpFilterSizerByFP = new wxBoxSizer( wxVERTICAL );
-    fpFilterSizerByFP->Add( m_filterByFPFilters, 0, wxLEFT | wxTOP | wxEXPAND, 5 );
-    bottomSizer->Add( fpFilterSizerByFP, 0, wxLEFT | wxTOP, 5 );
-
-    // buttonsSizer contains the m_filterByPinCount and BITMAP buttons
+    // buttonsSizer contains the BITMAP buttons
     wxBoxSizer* buttonsSizer = new wxBoxSizer( wxHORIZONTAL );
-    buttonsSizer->Add( m_filterByPinCount, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
 
-    // Add a spacer between the wxCheckBox and bitmap buttons
-    buttonsSizer->Add( 0, 0, 1, wxEXPAND, 5 );
+    buttonsSizer->Add( 0, 0, 1, 0, 5 );     // Add spacer to right-align buttons
 
-    m_grButton3DView = new BITMAP_BUTTON( bottomPanel, wxID_ANY,
-                                wxNullBitmap, wxDefaultPosition,
-                                wxDefaultSize/*, wxBU_AUTODRAW|wxBORDER_NONE*/ );
+    BITMAP_BUTTON* separator = new BITMAP_BUTTON( bottomPanel, wxID_ANY, wxNullBitmap );
+    separator->SetIsSeparator();
+    buttonsSizer->Add( separator, 0, wxRIGHT | wxALIGN_CENTER_VERTICAL, 1 );
+
+    m_grButton3DView = new BITMAP_BUTTON( bottomPanel, wxID_ANY, wxNullBitmap );
     m_grButton3DView->SetIsRadioButton();
     m_grButton3DView->SetBitmap( KiBitmapBundle( BITMAPS::shape_3d ) );
     m_grButton3DView->Check( !m_showFpMode );
-    buttonsSizer->Add( m_grButton3DView, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
+    buttonsSizer->Add( m_grButton3DView, 0, wxRIGHT | wxALIGN_CENTER_VERTICAL, 1 );
 
-    m_grButtonFpView = new BITMAP_BUTTON( bottomPanel, wxID_ANY,
-                                          wxNullBitmap, wxDefaultPosition,
-                                          wxDefaultSize/*, wxBU_AUTODRAW|wxBORDER_NONE*/ );
+    m_grButtonFpView = new BITMAP_BUTTON( bottomPanel, wxID_ANY, wxNullBitmap );
     m_grButtonFpView->SetIsRadioButton();
     m_grButtonFpView->SetBitmap( KiBitmapBundle( BITMAPS::module ) );
     m_grButtonFpView->Check( m_showFpMode );
-    buttonsSizer->Add( m_grButtonFpView, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
+    buttonsSizer->Add( m_grButtonFpView, 0, wxRIGHT | wxALIGN_CENTER_VERTICAL, 1 );
+
+    separator = new BITMAP_BUTTON( bottomPanel, wxID_ANY, wxNullBitmap );
+    separator->SetIsSeparator();
+    buttonsSizer->Add( separator, 0, wxRIGHT | wxALIGN_CENTER_VERTICAL, 1 );
+
+    m_show3DViewer = new wxCheckBox( bottomPanel, wxID_ANY, _( "Show 3D viewer in own window" ) );
+    buttonsSizer->Add( m_show3DViewer, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3 );
 
     wxStdDialogButtonSizer* sdbSizer = new wxStdDialogButtonSizer();
     wxButton*               okButton = new wxButton( bottomPanel, wxID_OK );
@@ -166,11 +161,9 @@ FOOTPRINT_CHOOSER_FRAME::FOOTPRINT_CHOOSER_FRAME( KIWAY* aKiway, wxWindow* aPare
     sdbSizer->AddButton( cancelButton );
     sdbSizer->Realize();
 
-    // Add a spacer between the bitmap buttons and thesdbSizer
-    buttonsSizer->Add( 0, 0, 1, wxEXPAND, 5 );
-    buttonsSizer->Add( sdbSizer, 1, wxALL, 5 );
-
-    bottomSizer->Add( buttonsSizer, 0, wxEXPAND | wxLEFT, 5 );
+    buttonsSizer->Add( 20, 0, 0, 0, 5 );     // Add spacer
+    buttonsSizer->Add( sdbSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
+    bottomSizer->Add( buttonsSizer, 0, wxEXPAND, 5 );
 
     bottomPanel->SetSizer( bottomSizer );
     frameSizer->Add( bottomPanel, 0, wxEXPAND );
@@ -183,28 +176,21 @@ FOOTPRINT_CHOOSER_FRAME::FOOTPRINT_CHOOSER_FRAME( KIWAY* aKiway, wxWindow* aPare
     Layout();
     m_chooserPanel->FinishSetup();
 
-    m_filterByPinCount->Bind( wxEVT_CHECKBOX,
-            [&]( wxCommandEvent& evt )
-            {
-                m_chooserPanel->Regenerate();
-            } );
-
-    m_filterByFPFilters->Bind( wxEVT_CHECKBOX,
-            [&]( wxCommandEvent& evt )
-            {
-                m_chooserPanel->Regenerate();
-            } );
-
     // Connect Events
     m_grButton3DView->Connect( wxEVT_COMMAND_BUTTON_CLICKED ,
-                         wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::on3DviewReq ),
-                         NULL, this );
+                               wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::on3DviewReq ),
+                               nullptr, this );
+
     m_grButtonFpView->Connect( wxEVT_COMMAND_BUTTON_CLICKED ,
-                             wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::onFpViewReq ),
-                             NULL, this );
+                               wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::onFpViewReq ),
+                               nullptr, this );
+
+    m_show3DViewer->Connect( wxEVT_COMMAND_CHECKBOX_CLICKED ,
+                             wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::onExternalViewer3DEnable ),
+                             nullptr, this );
 
     Connect( FP_SELECTION_EVENT,  // custom event fired by a PANEL_FOOTPRINT_CHOOSER
-             wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::onFpChanged ), NULL, this );
+             wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::onFpChanged ), nullptr, this );
 
     // Needed on Linux to fix the position of widgets in bottomPanel
     PostSizeEvent();
@@ -213,23 +199,115 @@ FOOTPRINT_CHOOSER_FRAME::FOOTPRINT_CHOOSER_FRAME( KIWAY* aKiway, wxWindow* aPare
 
 FOOTPRINT_CHOOSER_FRAME::~FOOTPRINT_CHOOSER_FRAME()
 {
+    // Work around assertion firing when we try to LockCtx on a hidden 3D canvas during dtor
+    wxCloseEvent dummy;
+    m_preview3DCanvas->Show();
+    m_preview3DCanvas->OnCloseWindow( dummy );
+
     // Disconnect Events
     m_grButton3DView->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED,
-                            wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::on3DviewReq ), NULL, this );
+                                  wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::on3DviewReq ),
+                                  nullptr, this );
     m_grButtonFpView->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED,
-                                wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::onFpViewReq ), NULL, this );
+                                  wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::onFpViewReq ),
+                                  nullptr, this );
+
+    m_show3DViewer->Disconnect( wxEVT_COMMAND_CHECKBOX_CLICKED ,
+                                wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::onExternalViewer3DEnable ),
+                                nullptr, this );
 
     Disconnect( FP_SELECTION_EVENT,
-                wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::onFpChanged ), NULL, this );
+                wxCommandEventHandler( FOOTPRINT_CHOOSER_FRAME::onFpChanged ), nullptr, this );
 
     if( PCBNEW_SETTINGS* cfg = dynamic_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() ) )
     {
-        cfg->m_FootprintChooser.use_fp_filters = m_filterByFPFilters->GetValue();
-        cfg->m_FootprintChooser.filter_on_pin_count = m_filterByPinCount->GetValue();
+        if( m_filterByFPFilters )
+            cfg->m_FootprintChooser.use_fp_filters = m_filterByFPFilters->GetValue();
+
+        if( m_filterByPinCount )
+            cfg->m_FootprintChooser.filter_on_pin_count = m_filterByPinCount->GetValue();
     }
 }
 
 
+void FOOTPRINT_CHOOSER_FRAME::onExternalViewer3DEnable( wxCommandEvent& aEvent )
+{
+    if( aEvent.IsChecked() )
+    {
+        if( m_grButton3DView->IsChecked() )
+            Show3DViewerFrame();        // show external 3D viewer
+    }
+    else
+    {
+        // Close the external 3D viewer frame, if it is still enabled
+        EDA_3D_VIEWER_FRAME* viewer3D = Get3DViewerFrame();
+
+        if( viewer3D )
+            viewer3D->Close( true );
+    }
+
+    updatePanelsVisibility();
+}
+
+
+void FOOTPRINT_CHOOSER_FRAME::Show3DViewerFrame()
+{
+    bool do_reload_board = true;    // reload board flag
+
+    // At EDA_3D_VIEWER_FRAME creation, the current board is loaded, so disable loading
+    // the current board if the 3D frame is not yet created
+    if( Get3DViewerFrame() == nullptr )
+        do_reload_board = false;
+
+    EDA_3D_VIEWER_FRAME* draw3DFrame = CreateAndShow3D_Frame();
+
+    // A stronger version of Raise() which promotes the window to its parent's level.
+    KIPLATFORM::UI::ReparentModal( draw3DFrame );
+
+    // And load or update the current board (if needed)
+    if( do_reload_board )
+        Update3DView( true, true );
+}
+
+
+void FOOTPRINT_CHOOSER_FRAME::Update3DView( bool aMarkDirty,
+                                            bool aRefresh, const wxString* aTitle )
+{
+    LIB_ID fpID = m_chooserPanel->GetSelectedLibId();
+    wxString footprintName;
+
+    if( fpID.IsValid() )
+        footprintName << fpID.Format();
+
+    wxString title = _( "3D Viewer" ) + wxT( " \u2014 " ) + footprintName;
+    PCB_BASE_FRAME::Update3DView( aMarkDirty, aRefresh, &title );
+}
+
+
+bool FOOTPRINT_CHOOSER_FRAME::filterByPinCount()
+{
+    if( m_filterByPinCount )
+        return m_filterByPinCount->GetValue();
+
+    if( PCBNEW_SETTINGS* cfg = dynamic_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() ) )
+        return cfg->m_FootprintChooser.filter_on_pin_count;
+
+    return false;
+}
+
+
+bool FOOTPRINT_CHOOSER_FRAME::filterByFPFilters()
+{
+    if( m_filterByFPFilters )
+        return m_filterByFPFilters->GetValue();
+
+    if( PCBNEW_SETTINGS* cfg = dynamic_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() ) )
+        return cfg->m_FootprintChooser.use_fp_filters;
+
+    return false;
+}
+
+
 bool FOOTPRINT_CHOOSER_FRAME::filterFootprint( LIB_TREE_NODE& aNode )
 {
     if( aNode.m_Type == LIB_TREE_NODE::TYPE::LIBRARY )
@@ -237,7 +315,7 @@ bool FOOTPRINT_CHOOSER_FRAME::filterFootprint( LIB_TREE_NODE& aNode )
         // Normally lib nodes get scored by the max of their children's scores.  However, if a
         // lib node *has* no children then the scorer will call the filter on the lib node itself,
         // and we just want to return true if we're not filtering at all.
-        return !m_filterByPinCount->GetValue() && !m_filterByFPFilters->GetValue();
+        return !filterByPinCount() && !filterByFPFilters();
     }
 
     auto patternMatch =
@@ -263,13 +341,13 @@ bool FOOTPRINT_CHOOSER_FRAME::filterFootprint( LIB_TREE_NODE& aNode )
                 return false;
             };
 
-    if( m_pinCount > 0 && m_filterByPinCount->GetValue() )
+    if( m_pinCount > 0 && filterByPinCount() )
     {
         if( aNode.m_PinCount != m_pinCount )
             return false;
     }
 
-    if( !m_fpFilters.empty() && m_filterByFPFilters->GetValue() )
+    if( !m_fpFilters.empty() && filterByFPFilters() )
     {
         if( !patternMatch( aNode.m_LibId, m_fpFilters ) )
             return false;
@@ -301,15 +379,18 @@ WINDOW_SETTINGS* FOOTPRINT_CHOOSER_FRAME::GetWindowSettings( APP_SETTINGS_BASE*
 
 COLOR_SETTINGS* FOOTPRINT_CHOOSER_FRAME::GetColorSettings( bool aForceRefresh ) const
 {
-    auto* settings = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
+    auto* cfg = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>( "fpedit" );
 
-    if( settings )
-        return Pgm().GetSettingsManager().GetColorSettings( settings->m_ColorTheme );
+    if( cfg )
+        return Pgm().GetSettingsManager().GetColorSettings( cfg->m_ColorTheme );
     else
         return Pgm().GetSettingsManager().GetColorSettings();
 }
 
 
+static wxRect s_dialogRect( 0, 0, 0, 0 );
+
+
 void FOOTPRINT_CHOOSER_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
 {
     const std::string& payload = mail.GetPayload();
@@ -318,6 +399,10 @@ void FOOTPRINT_CHOOSER_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
     {
     case MAIL_SYMBOL_NETLIST:
     {
+        wxSizer*  filtersSizer = m_chooserPanel->GetFiltersSizer();
+        wxWindow* filtersWindow = filtersSizer->GetContainingWindow();
+        wxString  msg;
+
         m_pinCount = 0;
         m_fpFilters.clear();
 
@@ -338,9 +423,17 @@ void FOOTPRINT_CHOOSER_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
 
             if( m_pinCount > 0 )
             {
-                m_filterByPinCount->SetLabel( m_filterByPinCount->GetLabel()
-                                                + wxString::Format( wxS( " (%d)" ), m_pinCount ) );
-                m_filterByPinCount->Show( true );
+                msg.Printf( _( "Filter by pin count (%d)" ), m_pinCount );
+                m_filterByPinCount = new wxCheckBox( filtersWindow, wxID_ANY, msg );
+
+                m_filterByPinCount->Bind( wxEVT_CHECKBOX,
+                        [&]( wxCommandEvent& evt )
+                        {
+                            m_chooserPanel->Regenerate();
+                        } );
+
+                if( PCBNEW_SETTINGS* cfg = dynamic_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() ) )
+                    m_filterByPinCount->SetValue( cfg->m_FootprintChooser.filter_on_pin_count );
             }
         }
 
@@ -352,12 +445,33 @@ void FOOTPRINT_CHOOSER_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
                 m_fpFilters.back()->SetPattern( filter.Lower() );
             }
 
-            m_filterByFPFilters->SetLabel( m_filterByFPFilters->GetLabel()
-                                            + wxString::Format( wxS( " (%s)" ), strings[1] ) );
-            m_filterByFPFilters->Show( true );
+            msg.Printf( _( "Apply footprint filters (%s)" ), strings[1] );
+            m_filterByFPFilters = new wxCheckBox( filtersWindow, wxID_ANY, msg );
+
+            m_filterByFPFilters->Bind( wxEVT_CHECKBOX,
+                    [&]( wxCommandEvent& evt )
+                    {
+                        m_chooserPanel->Regenerate();
+                    } );
+
+            if( PCBNEW_SETTINGS* cfg = dynamic_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() ) )
+                m_filterByFPFilters->SetValue( cfg->m_FootprintChooser.use_fp_filters );
         }
 
+        if( m_filterByFPFilters )
+            m_chooserPanel->GetFiltersSizer()->Add( m_filterByFPFilters, 0, wxEXPAND|wxBOTTOM, 4 );
+
+        if( m_filterByPinCount )
+            m_chooserPanel->GetFiltersSizer()->Add( m_filterByPinCount, 0, wxEXPAND|wxBOTTOM, 4 );
+
         m_chooserPanel->GetViewerPanel()->SetPinFunctions( pinNames );
+
+        // Save the wxFormBuilder size of the dialog...
+        if( s_dialogRect.GetSize().x == 0 || s_dialogRect.GetSize().y == 0 )
+            s_dialogRect = wxRect( wxWindow::GetPosition(), wxWindow::GetSize() );
+
+        // ... and then give it a kick to get it to layout the new items
+        GetSizer()->SetSizeHints( this );
         break;
     }
 
@@ -383,9 +497,6 @@ bool FOOTPRINT_CHOOSER_FRAME::ShowModal( wxString* aFootprint, wxWindow* aParent
 }
 
 
-static wxRect s_dialogRect( 0, 0, 0, 0 );
-
-
 void FOOTPRINT_CHOOSER_FRAME::SetPosition( const wxPoint& aNewPosition )
 {
     PCB_BASE_FRAME::SetPosition( aNewPosition );
@@ -474,17 +585,14 @@ void FOOTPRINT_CHOOSER_FRAME::closeFootprintChooser( wxCommandEvent& aEvent )
 
 void FOOTPRINT_CHOOSER_FRAME::onFpChanged( wxCommandEvent& event )
 {
-    if( m_showFpMode )      // the 3D viewer is not activated
-        return;
-
-    on3DviewReq( event );
+    updateViews();
 }
 
 
 void FOOTPRINT_CHOOSER_FRAME::build3DCanvas()
 {
     // Create the dummy board used by the 3D canvas
-    m_dummyBoard = new BOARD();
+    m_dummyBoard = GetBoard();
     m_dummyBoard->SetProject( &Prj(), true );
 
     // This board will only be used to hold a footprint for viewing
@@ -494,7 +602,9 @@ void FOOTPRINT_CHOOSER_FRAME::build3DCanvas()
     m_boardAdapter.m_IsBoardView = false;
     m_boardAdapter.m_IsPreviewer = true;   // Force display 3D models, regardless the 3D viewer options
 
-    EDA_3D_VIEWER_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<EDA_3D_VIEWER_SETTINGS>();
+    SETTINGS_MANAGER&       mgr = Pgm().GetSettingsManager();
+    EDA_3D_VIEWER_SETTINGS* cfg = mgr.GetAppSettings<EDA_3D_VIEWER_SETTINGS>( "3d_viewer" );
+
     m_boardAdapter.m_Cfg = cfg;
 
     // Build the 3D canvas
@@ -515,40 +625,93 @@ void FOOTPRINT_CHOOSER_FRAME::build3DCanvas()
 }
 
 
-
 void FOOTPRINT_CHOOSER_FRAME::on3DviewReq( wxCommandEvent& event )
 {
-    m_showFpMode = false;
+    if( m_show3DMode == true )
+    {
+        if( m_showFpMode == true )
+        {
+            m_show3DMode = false;
+            m_grButton3DView->Check( m_show3DMode );
+            updatePanelsVisibility();
+        }
+    }
+    else
+    {
+        if( m_show3DViewer->IsChecked() )
+        {
+            Show3DViewerFrame();
+        }
+        else
+        {
+            // Close 3D viewer frame, if it is still enabled
+            EDA_3D_VIEWER_FRAME* viewer3D = Get3DViewerFrame();
+            if( viewer3D )
+                viewer3D->Close( true );
+        }
 
-    m_grButtonFpView->Check( m_showFpMode );
-    m_grButton3DView->Check( !m_showFpMode );
-
-    FOOTPRINT_PREVIEW_WIDGET* viewFpPanel = m_chooserPanel->GetViewerPanel();
-    viewFpPanel->Show( m_showFpMode );
-    m_preview3DCanvas->Show( !m_showFpMode );
-    m_dummyBoard->DeleteAllFootprints();
-
-    if( m_chooserPanel->m_CurrFootprint )
-        m_dummyBoard->Add( (FOOTPRINT*)m_chooserPanel->m_CurrFootprint->Clone() );
-
-    m_preview3DCanvas->ReloadRequest();
-    m_preview3DCanvas->Request_refresh();
-    m_chooserPanel->m_RightPanel->Layout();
-    m_chooserPanel->m_RightPanel->Refresh();
+        m_show3DMode = true;
+        m_grButton3DView->Check( m_show3DMode );
+        updatePanelsVisibility();
+    }
 }
 
 
 void FOOTPRINT_CHOOSER_FRAME::onFpViewReq( wxCommandEvent& event )
 {
-    m_showFpMode = true;
+    if( m_showFpMode == true )
+    {
+        if( m_show3DMode == true )
+        {
+            m_showFpMode = false;
+            m_grButtonFpView->Check( m_showFpMode );
+            updatePanelsVisibility();
+        }
+    }
+    else
+    {
+        m_showFpMode = true;
+        m_grButtonFpView->Check( m_showFpMode );
+        updatePanelsVisibility();
+    }
+}
 
-    m_grButtonFpView->Check( m_showFpMode );
-    m_grButton3DView->Check( !m_showFpMode );
 
-    FOOTPRINT_PREVIEW_WIDGET* viewFpPanel = m_chooserPanel->GetViewerPanel();
-    viewFpPanel->Show( m_showFpMode );
-    m_preview3DCanvas->Show( !m_showFpMode );
+void FOOTPRINT_CHOOSER_FRAME::updateViews()
+{
+    EDA_3D_VIEWER_FRAME* viewer3D = Get3DViewerFrame();
+    bool reloadFp = viewer3D || m_preview3DCanvas->IsShown();
+
+    if( reloadFp )
+    {
+        m_dummyBoard->DeleteAllFootprints();
+
+        if( m_chooserPanel->m_CurrFootprint )
+            m_dummyBoard->Add( (FOOTPRINT*)m_chooserPanel->m_CurrFootprint->Clone() );
+
+    }
+
+    if( m_preview3DCanvas->IsShown() )
+    {
+        m_preview3DCanvas->ReloadRequest();
+        m_preview3DCanvas->Request_refresh();
+    }
+
+    if( viewer3D )
+    {
+        Update3DView( true, true );
+    }
+
     m_chooserPanel->m_RightPanel->Layout();
     m_chooserPanel->m_RightPanel->Refresh();
 }
 
+void FOOTPRINT_CHOOSER_FRAME::updatePanelsVisibility()
+{
+    FOOTPRINT_PREVIEW_WIDGET* viewFpPanel = m_chooserPanel->GetViewerPanel();
+    viewFpPanel->Show( m_showFpMode );
+    m_preview3DCanvas->Show( m_show3DMode );
+
+    updateViews();
+}
+
